// Copyright (c) 2009, Romain Tartière <romain@blogreen.org>
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following condition are met:
//
//    * Redistributions of source code must retain the above copyright notice,
//      this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright
//      notice, this list of conditions and the following disclaimer in the
//      documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

using System;
using Gtk;
using Tomboy;

namespace Tomboy.Todo {
	public class Todo : NoteAddin
	{
		/// <summary>
		/// Patterns we are looking for in notes.
		/// </summary>
		static string[] TODO_PATTERNS = {
			"FIXME",
			"TODO",
			"XXX"
		};

		/// <summary>
		/// Setup styles for highlighting patterns in notes.
		/// </summary>
		public override void Initialize()
		{
			foreach (string s in TODO_PATTERNS) {
				if (Note.TagTable.Lookup(s) == null) {
					TextTag tag = new TextTag(s);
					tag.Foreground = "#0080f0";
					tag.Weight = Pango.Weight.Bold;
					tag.Underline = Pango.Underline.Single;
					Note.TagTable.Add(tag);
				}
			}
		}

		public override void Shutdown()
		{
		}

		public override void OnNoteOpened()
		{
			Buffer.InsertText += OnInsertText;
			Buffer.DeleteRange += OnDeleteRange;
			
			HighlightNote();
		}

		private void OnInsertText(object sender, Gtk.InsertTextArgs args)
		{
			HighlightRegion(args.Pos, args.Pos);
		}
		
		private void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
		{
			HighlightRegion(args.Start, args.End);
		}

		private void HighlightNote()
		{
			HighlightRegion(Buffer.StartIter, Buffer.EndIter);
		}

		private void HighlightRegion(Gtk.TextIter start, Gtk.TextIter end)
		{
			if (!start.StartsLine())
				start.BackwardLine();
			if (!end.EndsLine())
				end.ForwardLine();
			
			foreach (string pattern in TODO_PATTERNS) {
				HighlightRegion(pattern, start, end);
			}		
		}

		private void HighlightRegion(string pattern, Gtk.TextIter start, Gtk.TextIter end)
		{
			Buffer.RemoveTag(pattern, start, end);
			Gtk.TextIter region_start = start;
			while (start.ForwardSearch(pattern + ":", TextSearchFlags.TextOnly, out region_start, out start, end)) {
				Gtk.TextIter region_end = start;
				//region_end.ForwardSentenceEnd();

				Buffer.ApplyTag(pattern, region_start, region_end);
			}
		}
	}
}
